function chimefind(varargin)
% chimefind(varargin)
%   Look for the "chimes" signifying the starts of utterances in
%   RATS dev0 transmissions; rewrite label times in seconds.
%   -clean <filename>     Path to clean (ref) wavfile
%   -labelsin <filename>  Input label file name
%   -labelsout <filename> Where to write 3-column output
%   -ldclabels 0/1        Input label file is 3 col <stt end lab> or 11 col
%   -segmap 0/1           Flag to both read and write 4-col segmap files
%   -chimeoffs <time>     Offset time added to chime (default 0.525 s)
%   -matchthresh <time>   Largest time difference to count as match (5 s)
%   -strict 0/1           Writes nothing if wrong number of chimes found (1)
%   -disp 0/1             Optional graphic display (0)
% 2011-05-18 Dan Ellis dpwe@ee.columbia.edu

VERSION = 0.7;
DATE = 20110804;

disp(['********** chimefind v',num2str(VERSION),' of ',num2str(DATE), ...
      ' **********']);

% Parse out the optional arguments
[CLEANIN, ...
 LABELSIN, LABELSOUT, ...
 CHIMEOFFS, ...
 MATCHTHRESH, ...
 STRICT, ...
 DISP, ...
 LDCLABELS, ...
 SEGMAP, ...
 DECHIMEOUT] = ...
    process_options(varargin, ...
                    '-clean', '', ...
		    '-labelsin', '', '-labelsout', '', ...
                    '-chimeoffs',  0.525, ...
                    '-matchthresh', 5.0, ...
                    '-strict', 1, ...
                    '-disp',  0, ...
                    '-ldclabels', 0, ...
                    '-segmap', 0, ...
                    '-dechimeout', '');

if length(CLEANIN)==0
  error('must specify -clean <wavfile>');
end
[p,n,e] = fileparts(CLEANIN);
% remove ".clean" extension too
[p,cleanstem,e] = fileparts(n);

% Read the input labels first, if any, so that if it dies here we won't
% have to wait for the audio processing
if length(LABELSIN) > 0
 
  % Read in the input label file
  if LDCLABELS || SEGMAP
    if LDCLABELS
      disp(['Reading LDC-format label file ', LABELSIN, ' ...']);
      % LDC2011E32 tab-separated columns
      % 1:filename 2:channel 3:starttime 4:endtime 5:timesource
      % 6:spkr 7:spkrsource 8:lang 9:langsource 10:transcript 11:transsource
      [fn,ch,ss,ee,tsrc,sid,ssrc,lid,lsrc,ll,lasrc] = textread(LABELSIN, ...
                             '%s\t%s\t%f\t%f\t%s\t%s\t%s\t%s\t%s\t%s\t%s', ...
                             'delimiter', '\t');
    else % SEGMAP
      disp(['Reading 4-column segmap file ', LABELSIN]);
      % compositefilename starttime endtime componentfilename
      [fn,ss,ee,ll] = textread(LABELSIN,'%s\t%f\t%f\t%s','delimiter','\t');
    end
    goodlines = strmatch(cleanstem,fn);
    ss = ss(goodlines);
    ee = ee(goodlines);
    ll = ll(goodlines);
    disp(['... found ', num2str(length(ss)), ' labels for ', cleanstem]);
  else
    disp(['Reading 3-column label file ', LABELSIN]);
    [ss,ee,ll] = textread(LABELSIN,'%f\t%f\t%s','delimiter','\t');
  end

else
  ss = [];
end

% Now do the audio processing...

% Read the wavefile
targetsr = 8000;
[d,sr] = audioread(CLEANIN,targetsr,1);
filedur = length(d)/sr;

% Locate the chimes
T = chimefindinwav(d,sr) + CHIMEOFFS;

nlabs = length(ss);
nchimes = length(T);



% Check consistency
%if nlabs > 0 && nlabs ~= length(T)
%  disp(['Warning: found ',num2str(length(T)),' chimes in ', CLEANIN, ...
%         ' but ',num2str(nlabs),' label segments in ', LABELSIN]);
%end

% Match labels & calculate new label times
if nlabs > 0
  % We read an input label file; have ss, ee, etc.
  dd = ee - ss;
  
  if STRICT
    % STRICT mode simply matches up labels in sequence
    ix = 1:min(nchimes,nlabs);
    minabsdt = abs(T(ix)-ss(ix));
  else
    dtimes = repmat(ss,1,nchimes) - repmat(T',nlabs,1);
    [minabsdt,ix] = min(abs(dtimes));
  end
  
  founddts = find(minabsdt<MATCHTHRESH);
  ix = ix(founddts);
  nfound = length(founddts);
    
  skews = T(founddts) - ss(ix);
  disp(['Matched ',num2str(nfound),' of ', ...
        num2str(nchimes),' chimes (within ', ...
        num2str(MATCHTHRESH), ' secs)']);

  disp(['  median chime delay = ',num2str(median(skews)),...
        ' s, min = ', num2str(min(skews)), ...
        ' s, max = ', num2str(max(skews)), ' s']);
  
  news = T(founddts);
  nuniqlabs = length(unique(ix));
  if nuniqlabs ~= nfound
    disp(['*** Only ',num2str(nuniqlabs),' unique labels in the ', ...
          num2str(nfound),' matches']);
  end
  if nlabs ~= nfound
      disp(['*** Matched only ',num2str(nfound),...
            ' chimes in ', CLEANIN, ...
            ' to ',num2str(nlabs),' label segments in ', LABELSIN]);
  end
  
  unusedlabs = ones(1,length(ss));
  unusedlabs(ix) = 0;
  unusedlabs = find(unusedlabs);
  if length(unusedlabs) > 0
    disp([num2str(length(unusedlabs)),' unused labels:']);
    for i = 1:length(unusedlabs)
      ts = round(ss(unusedlabs(i)));
      fprintf('  #%d: %.3f (%d:%02d) %.3f %s\n',unusedlabs(i), ...
              ss(unusedlabs(i)), floor(ts/60), rem(ts, 60), ...
              ee(unusedlabs(i)),ll{unusedlabs(i)});
    end
  end
  
  if STRICT
    if nlabs ~= nfound || nuniqlabs ~= length(ix)
      disp(['Failed STRICT test of ',num2str(nchimes), ...
            ' chimes matching ',num2str(nlabs), ...
            ' labels - writing nothing']);
      news = [];
      ix = [];
    end
  end
  ll = ll(ix);
  ss = ss(ix);
  ee = ee(ix);
  dd = dd(ix);
  % add segment lengths onto start times
  newe = news + dd;

else
  disp('No input labels provided; outputting pure times');
  news = T;
  % end each segment 5s before the next one starts
  newe = [T(2:end);filedur] - 5.0;
  for i = 1:length(news)
    ll{i} = '';
  end
end

% Write output segments
if length(LABELSOUT) > 0 
  if length(news) > 0
    f = fopen(LABELSOUT,'w');
    if SEGMAP % 4-column filename start end label
      for i = 1:length(news)
        fprintf(f,'%s\t%.3f\t%.3f\t%s\n',cleanstem,news(i),newe(i),ll{i});
      end
      fmt = 'segmap';
    else % 3-column start end label
      for i = 1:length(news)
        fprintf(f,'%.3f\t%.3f\t%s\n',news(i),newe(i),ll{i});
      end
      fmt = 'audacity';
    end
    fclose(f);
    disp([num2str(length(news)),' adjusted labels written to ', ...
          LABELSOUT, ' (',fmt,' format)']);
  else
    disp(['** No matches - not writing ',LABELSOUT]);
  end
end

% Maybe plot
if DISP
  subplot(211)
  %specgram(d,512,sr);
  %colormap(gray);
  plot([1:length(d)]/sr,d);
  % Overplot the times
  hold on;
  plot([1;1]*T',[-1;1]*ones(1,length(T)),'-r'); 
  hold off;
  [p,n,e] = fileparts(CLEANIN);
  title(['Chimes within ',n],'interpreter','none');
  axis([0 filedur -1 1]);
  xlabel('time / s');
  
  % Plot the skews if we have them
  if length(ss) > 0
    subplot(212)
    plot(news,news-ss,'.');
    grid
    [p,n,e] = fileparts(LABELSIN);
    title(['Label offsets to ',n],'interpreter','none');
    xlabel('Chime-based segment start / s');
    ylabel('Delay of chime-based start rel. to label file');
  end
end

% Maybe rewrite de-chimed audio?
if length(DECHIMEOUT) > 0
  % Re-read the original audio
  [d,sr] = audioread(CLEANIN,targetsr,1);
  % replace 350ms around identified chimes with low-level noise
  chimelen = round(sr * 0.350);
  for i = 1:length(T)
    % back up 10ms to be sure
    tpos = max(0, round(sr*(T(i)-CHIMEOFFS-0.010)));
    d(tpos + [1:chimelen]) = 0.0003 * randn(chimelen,1);
  end
  % rewrite the audio
  audiowrite(d,sr,DECHIMEOUT);
  disp(['De-chimed audio written to ',DECHIMEOUT]);
end
